var path = require("path");
var fileAndFolderUtils = require('../file-and-folder-utils');
var consoleFileLogger = require("../sl-logger.js").consoleFileLogger;
var utils = require('../utils');
var SlJsInfra = require('sl-js-infra').SlJsInfra;
var CiaEnvVars = SlJsInfra.SlEnvVars.CIA;

var THREE_SLASHES_WEBPACK = 'webpack:///';
var ONE_SLASH_WEBPACK = "webpack:/";

var SourceFileResolver ={};

SourceFileResolver.isResolved = false;
SourceFileResolver.sourceRoot = "";


function extractSourceRoot(givenSource, sourcePath, log) {
    var normalizedGiven = normalizePath(givenSource);
    var normalizedSource = normalizePath(sourcePath);
    var trimmed = fileAndFolderUtils.removeRelativeSigns(normalizedGiven);
    var sourceRoot = normalizedSource.replace(trimmed, "");
    log.info("found source in path: '%s'", sourceRoot);
    SourceFileResolver.sourceRoot = sourceRoot;
    SourceFileResolver.isResolved = true;
    return SourceFileResolver.sourceRoot;
}

function normalizePath(filePath){
    var normalized = path.normalize(filePath);
    return utils.adjustPathSlashes(normalized);
}

function trimPackerPrefix(fileName) {
    fileName = utils.adjustPathSlashes(fileName);
    if (fileName.indexOf(THREE_SLASHES_WEBPACK) === 0) {
        fileName = fileName.substring(THREE_SLASHES_WEBPACK.length);
    }
    else if (fileName.indexOf(ONE_SLASH_WEBPACK) >= 0){
        var idx = fileName.indexOf(ONE_SLASH_WEBPACK);
        //Get just the filename (without the packer)
        fileName = fileName.substring(idx + ONE_SLASH_WEBPACK.length);
    }
    return fileName;
}


SourceFileResolver.resolveSourceRoot = function (givenSource, mapFileDir, logger) {
    var log = logger ? logger.child({
        methodName: "resolveSourceRoot"
    }) : consoleFileLogger("resolveSourceRoot");
    if(givenSource.indexOf("node_modules") > -1){
        return mapFileDir;
    }
    if (SourceFileResolver.sourceRoot) {
        return SourceFileResolver.sourceRoot;
    }
    if(process.env.SL_IS_REACT){
        SourceFileResolver.sourceRoot = mapFileDir;
        SourceFileResolver.isResolved = true;
        return SourceFileResolver.sourceRoot;
    }
    if (CiaEnvVars.getSourceRoot() != null) {
        SourceFileResolver.sourceRoot = CiaEnvVars.getSourceRoot();
        SourceFileResolver.isResolved = true;
        return SourceFileResolver.sourceRoot;
    }
    givenSource = trimPackerPrefix(givenSource);
    var sourcePath = fileAndFolderUtils.findFileRecursively(givenSource, mapFileDir, true, logger);
    if(sourcePath){
        return extractSourceRoot(givenSource, sourcePath, log);
    }
    return mapFileDir;


};

module.exports = SourceFileResolver;
